{% extends "base.html" %}

{% block script %}
    {{super()}}
    <script type="text/javascript">
        var hostname = '{{hostname}}';
        var version = '{{version}}';
    </script>
    <script type="text/javascript" src="{{static_path}}js/version.js?v={{version}}"></script>
{% endblock %}

{% block body %}
hostname = "{{hostname}}"<br>
version = "{{version}}"<br>
motion_version = "{{motion_version}}"<br>
os_version = "{{os_version}}"
{% endblock %}
