<!DOCTYPE html>
<html>
    <head>
        {% block meta %}
            <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
            <meta http-equiv="X-UA-Compatible" content="IE=Edge">
            <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no">
            <meta name="mobile-web-app-capable" content="yes">
            <meta name="apple-mobile-web-app-capable" content="yes">
            <meta name="theme-color" content="#414141">
            <meta name="apple-mobile-web-app-status-bar-style" content="#414141">
        {% endblock %}
        <title>{% block title %}{% endblock %}</title>
        {% block style %}
            <link rel="stylesheet" type="text/css" href="{{static_path}}css/jquery.timepicker.css?v={{version}}" />
            <link rel="shortcut icon" href="{{static_path}}favicon.ico?v={{version}}" />
            <link rel="apple-touch-icon" href="{{static_path}}favicon.ico?v={{version}}" />
            <link rel="manifest" href="{{static_path}}../manifest.json?v={{version}}" />
        {% endblock %}
        {% block script %}
            <script type="text/javascript" src="{{static_path}}js/css-browser-selector.js?v={{version}}"></script>
            <script type="text/javascript" src="{{static_path}}js/jquery.min.js?v={{version}}"></script>
            <script type="text/javascript" src="{{static_path}}js/jquery.timepicker.min.js?v={{version}}"></script>
            <script type="text/javascript" src="{{static_path}}js/jquery.mousewheel.js?v={{version}}"></script>

            <script type="text/javascript">
                var staticPath = '{{static_path}}';
            </script>
        {% endblock %}
    </head>
    
    <body>
        {% block body %}{% endblock %}
    </body>
</html>
